#!/usr/bin/python
from unicode_map import glyphs as requested_glyphs
from sys import stdin,stdout,stderr,exit
print("%d glyphs"%len(requested_glyphs))

def binary_image_data(inp):
    bmap = {
        '0': [0,0,0,0], '1': [0,0,0,1], '2': [0,0,1,0], '3': [0,0,1,1],
        '4': [0,1,0,0], '5': [0,1,0,1], '6': [0,1,1,0], '7': [0,1,1,1],
        '8': [1,0,0,0], '9': [1,0,0,1], 'A': [1,0,1,0], 'B': [1,0,1,1],
        'C': [1,1,0,0], 'D': [1,1,0,1], 'E': [1,1,1,0], 'F': [1,1,1,1]      }
    return sum( map(lambda i: bmap[i], inp), [] )

requested_glyphs_set = set(requested_glyphs)
glyphs = {}

for line in stdin:
    codept,glyph = line.strip().split(':')
    codept = int(codept, 16)

    if codept in requested_glyphs_set:
        if len(glyph) != 32:
            raise Exception("%d is wide glyph"%codept)
        glyphs[codept] = glyph
        requested_glyphs_set.remove(codept)

# exit if there are any missing requested glyphs
if len(requested_glyphs_set) != 0:
    print("Error: These unicode codepoints do not have glyphs:")
    print(requested_glyphs_set)
    exit(1)

# make sure that the number of glyphs is a multiple of four.
# if it's not, add blank glyphs
while len(requested_glyphs) % 4:
    requested_glyphs.append(ord(' '))

fout = open("UNIFONT.BIN","wb")
fout.write(bytearray([0,0])) # commodore header

while requested_glyphs:
    cur_glyphs = requested_glyphs[0:4]
    requested_glyphs = requested_glyphs[4:]
    # cur_glyphs are the four glyphs that will be combined in this step.
    # their order is reversed so that bits can be pushed in from the right
    cur_glyphs.reverse()
    cur_glyphs = list(map(lambda x: binary_image_data(glyphs[x]), cur_glyphs))

    # each byte written to the output is made up of two pixels from each of
    # the four glyphs
    while cur_glyphs[0]:
        byte = 0
        for repeat in [0,1]:
            for im in cur_glyphs:
                byte = (byte<<1) + im[0]
                del im[0]
        fout.write(bytearray([byte]))

fout.close()
