#!/usr/bin/python
import sys

# list of glyphs that will be packaged. each item in list is a 16-bit unicode
# code point. its index in the list is its screen code.
glyphs = [ ]

# dictionary of characters that are accessible. keys are 16-bit unicode code
# points, values are 16-bit tile data that can be pushed into vera
unicode_map = { }

# adds a given unicode character whose glyph is directly loaded
def add_glyph(codept):
    global glyphs, unicode_map

    if codept in unicode_map:
        return # skip duplicates

    screen_code = len(glyphs)
    glyphs.append(codept)
    if screen_code > 4096:
        raise Exception("error: too many glyphs")

    # convert this into vera tile data
    ind_palette = screen_code & 0x3
    ind_lo = (screen_code>>2) & 0xff
    ind_hi = (screen_code>>10)

    unicode_map[codept] = ind_lo | (ind_hi<<8) | (ind_palette<<12)

# adds a new unicode code point without adding a new glyph. codept is the new
# code point, codept_src is the code point to copy the glyph from. 
FLIPH=2
FLIPV=1
FLIP=3
def add_glyph_copy(codept, codept_src, flip=0):
    global unicode_map
    if codept in unicode_map:
        raise Exception("duplicate codepoint")
    unicode_map[codept] = unicode_map[codept_src]^(flip<<10)

#####
# all the characters we want to include
#####

# 0: space, 1: full block
add_glyph(ord(' '))
add_glyph(0x2588)

# printable ascii characters
for i in range(0x20, 0x7f):
    add_glyph(i)

# 00 latin-1 supplement
add_glyph_copy(0xa0, ord(' '))          # non breaking space
add_glyph_copy(0xa1, ord('!'), FLIPH)   # inverted exclamation mark
for i in range(0xa2,0xad): add_glyph(i)
add_glyph_copy(0xad, ord('-'))          # soft hyphen
for i in range(0xae, 0xbf): add_glyph(i)
add_glyph_copy(0xbf, ord('?'), FLIP)    # inverted question mark
add_glyph(0xc0)                         # capital A with grave
add_glyph_copy(0xc1, 0xc0, FLIPV)       # capital A with acute
for i in range(0xc2, 0xcc): add_glyph(i)
add_glyph(0xcc)                         # capital I with grave
add_glyph_copy(0xcd, 0xcc, FLIPV)       # capital I with acute
for i in range(0xce, 0xd2): add_glyph(i)
add_glyph(0xd2)                         # capital O with grave
add_glyph_copy(0xd3, 0xd2, FLIPV)       # capital O with acute
for i in range(0xd4, 0xd9): add_glyph(i)
add_glyph(0xd9)                         # capital U with grave
add_glyph_copy(0xda, 0xd9, FLIPV)       # capital U with acute
for i in range(0xdb, 0xec): add_glyph(i)
add_glyph(0xec)                         # minisc I with grave
add_glyph_copy(0xed, 0xec, FLIPV)       # minisc I with acute
for i in range(0xee, 0xf2): add_glyph(i)
add_glyph(0xf2)                         # minisc O with grave
add_glyph_copy(0xf3, 0xf2, FLIPV)       # minisc O with acute
for i in range(0xf4, 0x100): add_glyph(i)

# 01 latin extended-a
# TODO there's probably a few acute/grave accents on symmetric letters
# that get duplicated
for i in range(0x0100,0x0149):
    add_glyph(i)
for i in range(0x014a,0x0180):
    add_glyph(i)

# 03 greek & coptic
for i in range(0x370, 0x374): add_glyph(i)
add_glyph(0x374)                        # greek numeral sign
add_glyph_copy(0x375, 0x374, FLIPH)     # greek lower numeral sign
add_glyph_copy(0x376, ord('N'), FLIPV)  # capital pamphylian digamma
add_glyph(0x377)                        # minisc idem
add_glyph(0x37a)
add_glyph(0x37b)
add_glyph(0x37c)
add_glyph_copy(0x378, 0x37c, FLIPV)
add_glyph_copy(0x37e, ord(';'))         # greek question mark (is this okay?)
add_glyph_copy(0x37f, ord('J'))         # capital yot
for i in range(0x384, 0x391):
    if i not in [0x38b,0x38d]:
        add_glyph(i)
add_glyph_copy(0x391, ord('A'))         # capital alpha
add_glyph_copy(0x392, ord('B'))         # capital beta
add_glyph(0x393)
add_glyph(0x394)
add_glyph_copy(0x395, ord('E'))         # capital epsilon
add_glyph_copy(0x396, ord('Z'))         # capital zeta
add_glyph_copy(0x397, ord('H'))         # capital eta
add_glyph(0x398)
add_glyph_copy(0x399, ord('I'))         # capital iota
add_glyph_copy(0x39a, ord('K'))         # capital kappa
add_glyph(0x39b)
add_glyph_copy(0x39c, ord('M'))         # capital mu
add_glyph_copy(0x39d, ord('N'))         # capital nu
add_glyph(0x39e)
add_glyph_copy(0x39f, ord('O'))         # capital omicron
add_glyph(0x3a0)
add_glyph_copy(0x3a1, ord('P'))         # capital rho
add_glyph(0x3a3)
add_glyph_copy(0x3a4, ord('T'))         # capital tau
add_glyph_copy(0x3a5, ord('Y'))         # capital upsilon
add_glyph(0x3a6)
add_glyph_copy(0x3a7, ord('X'))         # capital chi
add_glyph(0x3a8)
add_glyph(0x3a9)
add_glyph_copy(0x3aa, 0xcf)             # capital iota with diaer
for i in range(0x3ab, 0x400): add_glyph(i)

# 04 cyrillic
add_glyph_copy(0x0400,0x00c8)
add_glyph_copy(0x0401,0x00cb)
add_glyph(0x0402)
add_glyph(0x0403)
add_glyph(0x0404)
add_glyph_copy(0x0405,ord('S'))
add_glyph_copy(0x0406,ord('I'))
add_glyph_copy(0x0407,0x00cf)
add_glyph_copy(0x0408,ord('J'))
for i in range(0x0409,0x0410):
    add_glyph(i)
add_glyph_copy(0x0410,ord('A'))
add_glyph(0x0411)
add_glyph_copy(0x0412,ord('B'))
add_glyph(0x0413)
add_glyph(0x0414)
add_glyph_copy(0x0415,ord('E'))
add_glyph(0x0416)
add_glyph(0x0417)
add_glyph_copy(0x0418,ord('N'),FLIPV)
add_glyph(0x0419)
add_glyph_copy(0x041a,ord('K'))
add_glyph(0x041b)
add_glyph_copy(0x041c,ord('M'))
add_glyph_copy(0x041d,ord('H'))
add_glyph_copy(0x041e,ord('O'))
add_glyph_copy(0x041f,0x03a0)
add_glyph_copy(0x0420,ord('P'))
add_glyph_copy(0x0421,ord('C'))
add_glyph_copy(0x0422,ord('T'))
add_glyph(0x0423)
add_glyph(0x0424)
add_glyph_copy(0x0425,ord('X'))
for i in range(0x0426,0x042d):
    add_glyph(i)
add_glyph_copy(0x042d,0x0400,FLIPV)
add_glyph(0x042e)
add_glyph_copy(0x042f,ord('R'),FLIPV)
add_glyph_copy(0x0430,ord('a'))
add_glyph(0x0431)
add_glyph(0x0432)
add_glyph(0x0433)
add_glyph(0x0434)
add_glyph_copy(0x0435,ord('e'))
add_glyph(0x0436)
add_glyph_copy(0x0437,0x03b5,FLIPV)
add_glyph_copy(0x0438,0x0377)
add_glyph(0x0439)
add_glyph_copy(0x043a,ord('k'))
add_glyph(0x043b)
add_glyph(0x043c)
add_glyph(0x043d)
add_glyph_copy(0x043e,ord('o'))
add_glyph(0x043f)
add_glyph_copy(0x0440,ord('p'))
add_glyph_copy(0x0441,ord('c'))
add_glyph(0x0442)
add_glyph_copy(0x0443,ord('y'))
add_glyph(0x0444)
add_glyph_copy(0x0445,ord('x'))
for i in range(0x0446,0x044d):
    add_glyph(i)
add_glyph_copy(0x044d,0x03f6)
add_glyph(0x044e)
add_glyph(0x044f)
add_glyph_copy(0x0450,0x00e8)
add_glyph_copy(0x0451,0x00eb)
add_glyph(0x0452)
add_glyph(0x0453)
add_glyph_copy(0x0454,0x03f5)
add_glyph_copy(0x0455,ord('s'))
add_glyph_copy(0x0456,ord('i'))
add_glyph_copy(0x0457,0x00ef)
add_glyph_copy(0x0458,ord('j'))
for i in range(0x0459,0x0483):
    add_glyph(i)
for i in range(0x048a,0x04d2):
    add_glyph(i)
add_glyph_copy(0x04d2,0x00c4)
add_glyph_copy(0x04d3,0x00e4)
add_glyph_copy(0x04d4,0x00c6)
add_glyph_copy(0x04d5,0x00e6)
for i in range(0x04d6,0x04d9):
    add_glyph(i)
add_glyph_copy(0x04d9,ord('e'),FLIP)
for i in range(0x04da,0x0500):
    add_glyph(i)

# 25 box drawing
add_glyph(0x2500)                       # hor and vert lines
add_glyph_copy(0x2501,0x2500)
add_glyph(0x2502)
add_glyph_copy(0x2503,0x2502)
add_glyph(0x2504)
add_glyph_copy(0x2505,0x2504)
add_glyph(0x2506)
add_glyph_copy(0x2507,0x2506)
for i in range(0x2508,0x250c):
    add_glyph_copy(i,i-4)
add_glyph(0x250c)                       # elbow pieces
for i in range(0x250d,0x2510):
    add_glyph_copy(i,0x250c)
for i in range(0x2510,0x2514):
    add_glyph_copy(i,0x250c,FLIPV)
for i in range(0x2514,0x2518):
    add_glyph_copy(i,0x250c,FLIPH)
for i in range(0x2518,0x251c):
    add_glyph_copy(i,0x250c,FLIP)
add_glyph(0x251c)                       # T shapes
for i in range(0x251d,0x2524):
    add_glyph_copy(i,0x251c)
for i in range(0x2524,0x252c):
    add_glyph_copy(i,0x251c,FLIP)
add_glyph(0x252c)
for i in range(0x252d,0x2534):
    add_glyph_copy(i,0x252c)
for i in range(0x2534,0x253c):
    add_glyph_copy(i,0x252c,FLIP)
add_glyph(0x253c)                       # t shape
for i in range(0x253d,0x254c):
    add_glyph_copy(i,0x253c)
add_glyph_copy(0x254c,0x2504)
add_glyph_copy(0x254d,0x2504)
add_glyph_copy(0x254e,0x2506)
add_glyph_copy(0x254f,0x2506)
add_glyph(0x2550)
add_glyph(0x2551)
add_glyph(0x2552)
add_glyph(0x2553)
add_glyph(0x2554)
add_glyph_copy(0x2555,0x2552,FLIPV)
add_glyph_copy(0x2556,0x2553,FLIPV)
add_glyph_copy(0x2557,0x2554,FLIPV)
add_glyph_copy(0x2558,0x2552,FLIPH)
add_glyph_copy(0x2559,0x2553,FLIPH)
add_glyph_copy(0x255a,0x2557,FLIP)
add_glyph_copy(0x255b,0x2558,FLIPV)
add_glyph_copy(0x255c,0x2559,FLIPV)
add_glyph_copy(0x255d,0x255a,FLIPV)
for i in range(0x255e,0x2561):
    add_glyph(i)
add_glyph_copy(0x2561,0x255e,FLIPV)
add_glyph_copy(0x2562,0x255f,FLIPV)
add_glyph_copy(0x2563,0x2560,FLIPV)
add_glyph(0x2564)
add_glyph(0x2565)
add_glyph(0x2566)
add_glyph_copy(0x2567,0x2564,FLIPH)
add_glyph_copy(0x2568,0x2565,FLIPH)
add_glyph_copy(0x2569,0x2566,FLIPH)
add_glyph(0x256a)
add_glyph(0x256b)
add_glyph(0x256c)
add_glyph(0x256d)
add_glyph_copy(0x256e,0x256d,FLIPV)
add_glyph_copy(0x256f,0x256d,FLIP)
add_glyph_copy(0x2570,0x256e,FLIPH)
add_glyph(0x2571)
add_glyph_copy(0x2572,0x2571,FLIPH)
add_glyph(0x2573)
add_glyph(0x2574)
add_glyph(0x2575)
add_glyph_copy(0x2576,0x2574,FLIP)
add_glyph_copy(0x2577,0x2575,FLIP)
for i in range(0x2578,0x257c):
    add_glyph_copy(i,i-4)
add_glyph_copy(0x257c,0x2500)
add_glyph_copy(0x257d,0x2502)
add_glyph_copy(0x257e,0x2500)
add_glyph_copy(0x257f,0x2502)

# 25 block elements
for i in range(0x2580,0x2590):
    add_glyph(i)
add_glyph_copy(0x2590,0x258c,FLIP)
add_glyph(0x2591)
add_glyph(0x2592)
add_glyph_copy(0x2593,0x2592)
add_glyph_copy(0x2594,0x2581,FLIP)
add_glyph_copy(0x2595,0x258f,FLIP)
add_glyph(0x2596)
add_glyph_copy(0x2597,0x2596,FLIPV)
add_glyph_copy(0x2598,0x2596,FLIPH)
add_glyph(0x2599)
add_glyph(0x259a)
add_glyph_copy(0x259b,0x2599,FLIPH)
add_glyph_copy(0x259c,0x2599,FLIP)
add_glyph_copy(0x259d,0x2596,FLIP)
add_glyph_copy(0x259e,0x259a,FLIPV)
add_glyph_copy(0x259f,0x259b,FLIP)
