#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <termios.h>
#include <sys/ioctl.h>
void handleInput(int);
void handleOutput(int);

int main(void) {
    int pt_master;
    int pt_slave;
    char *name;

    pt_master = posix_openpt(O_RDWR);
    grantpt(pt_master);
    unlockpt(pt_master);
    name = ptsname(pt_master);
    pt_slave = open(name, O_RDWR);

    if (fork() == 0) {
        struct winsize ws;
        /* child process */
        setsid();
        close(pt_master);
        /* set winsize */
        ws.ws_row=30;
        ws.ws_col=80;
        ioctl(pt_slave, TIOCSWINSZ, &ws);
        /* setup stdio */
        dup2(pt_slave, 0);
        dup2(pt_slave, 1);
        dup2(pt_slave, 2);
        close(pt_slave);
        setenv("TERM","xterm",1);
        /* TODO pretending to be xterm is a bad idea. */
        execl("/bin/bash", "/bin/bash", NULL);
    }

    /* parent process */
    close(pt_slave);
    /* fork again-- one handles input, the other handles output */
    if (fork() == 0) {
        handleOutput(pt_master);
    } else {
        handleInput(pt_master);
    }
    return 0;
}

/* handles input from emulated x16 */
void handleInput(int fd) {
    char buffer[4];
    int f;

    while (1) {
        read(fd, buffer, 1); 

        do {
            /* wait for file STDIN to be cleared */
            f = open("STDIN", O_RDONLY);
            read(f, buffer+1, 3);
            close(f);
        } while ( buffer[3] != 0 );

        /* once the pipe is cleared, write the next byte */
        f = open("STDIN", O_WRONLY);
        buffer[3] = buffer[0];
        write(f, buffer+1, 3);
        close(f);
    }
}

/* handles output from emulated x16 */
void handleOutput(int fd) {
    char buffer[4];
    int f;
    
    while (1) {
        /* wait for something to happen in STDOUT */
        buffer[2] = 0;
        do {
            f = open("STDOUT", O_RDONLY);
            read(f, buffer, 3);
            close(f);
        } while ( buffer[2] == 0 );
        /* once there's a character, process it */
        write(fd, buffer+2, 1);
        /* then write a zero back */
        buffer[2] = 0;
        f = open("STDOUT", O_WRONLY);
        write(f, buffer, 3);
        close(f);
    }
}
