; routines for executing ansi control sequences

; csi dispatch table. lookup routine to execute a control sequence by the csi
; final byte. 16-bit entries for bytes in the range $40-7f
; 
; for final byte B, address is found at (B-$40)<<1
.align 256
terminal_csi_dispatch:
    .word _null                                         ; @
    .word terminal_csi_cursor_up                        ; A
    .word terminal_csi_cursor_down                      ; B
    .word terminal_csi_cursor_forward                   ; C
    .word terminal_csi_cursor_back                      ; D
    .word terminal_csi_cursor_next_line                 ; E
    .word terminal_csi_cursor_prev_line                 ; F
    .word terminal_csi_cursor_hor_abs                   ; G
    .word terminal_csi_cursor_position                  ; H
    .word _null                                         ; I
    .word terminal_csi_erase_in_display                 ; J
    .word terminal_csi_erase_in_line                    ; K
    .word _null                                         ; L
    .word _null                                         ; M
    .word _null                                         ; N
    .word _null                                         ; O
    .word _null                                         ; P
    .word _null                                         ; Q
    .word _null                                         ; R
    .word terminal_csi_scroll_up                        ; S
    .word terminal_csi_scroll_down                      ; T
    .word _null                                         ; U
    .word _null                                         ; V
    .word _null                                         ; W
    .word _null                                         ; X
    .word _null                                         ; Y
    .word _null                                         ; Z
    .word _null                                         ; [
    .word _null                                         ; \
    .word _null                                         ; ]
    .word _null                                         ; ^
    .word _null                                         ; _
    .word _null                                         ; `
    .word _null                                         ; a
    .word _null                                         ; b
    .word _null                                         ; c
    .word terminal_csi_cursor_vert_abs                  ; d
    .word _null                                         ; e
    .word _null                                         ; f
    .word _null                                         ; g
    .word _null                                         ; h
    .word _null                                         ; i
    .word _null                                         ; j
    .word _null                                         ; k
    .word _null                                         ; l
    .word terminal_csi_set_graphics                     ; m
    .word _null                                         ; n
    .word _null                                         ; o
    .word _null                                         ; p
    .word _null                                         ; q
    .word _null                                         ; r
    .word _null                                         ; s
    .word _null                                         ; t
    .word _null                                         ; u
    .word _null                                         ; v
    .word _null                                         ; w
    .word _null                                         ; x
    .word _null                                         ; y
    .word _null                                         ; z
    .word _null                                         ; {
    .word _null                                         ; |
    .word _null                                         ; }
    .word _null                                         ; ~
    .word _null                                         ; DELETE
_null:
    rts ; for unsupported final bytes

; cursor up (CUU)
terminal_csi_cursor_up:
    lda terminal_parse_v
    bne +
        ina ; zero becomes one
+   eor #$ff    ; negate
    ina
    jmp terminal_cursor_vert_rel

; cursor down (CUD)
terminal_csi_cursor_down:
    lda terminal_parse_v
    bne +
        ina ; zero becomes one
+   jmp terminal_cursor_vert_rel

; cursor forward (CUF)
terminal_csi_cursor_forward:
    ; get current cursor position and increase it
    lda vera_addr_lo
    lsr
    bit terminal_parse_v
    bne +
        ; if parameter is zero, make it a one
        ina
+
    sec ; increment by one because terminal_cursor_hor is 1-indexed
    adc terminal_parse_v
    jmp terminal_cursor_hor

; cursor back (CUB)
terminal_csi_cursor_back:
    ; reuse CUF routine
    lda terminal_parse_v
    bne + ; replace 0 with 1 right away
        lda #1
+   eor #$ff
    ina
    sta terminal_parse_v
    jmp terminal_csi_cursor_forward

; cursor next line (CNL)
terminal_csi_cursor_next_line:
    ; this is the same as CUD, but additionally the cursor is moved
    ; to the beginning of the line
    stz vera_addr_lo
    jmp terminal_csi_cursor_down

; cursor previous line (CPL)
terminal_csi_cursor_prev_line:
    ; as with CNL, move to the beginning of the line and do a CUU
    stz vera_addr_lo
    jmp terminal_csi_cursor_up

; cursor horizontal absolute (CHA)
terminal_csi_cursor_hor_abs:
    lda terminal_parse_v
    jmp terminal_cursor_hor

; cursor position (CUP)
terminal_csi_cursor_position:
    ; parameters are row;column
    lda terminal_parse_v    ; row
    jsr terminal_cursor_vert
    lda #0
    ; make sure there is a second argument before looking at it
    ldy terminal_parse_v_count
    beq +  ; if there isn't; set it to zero
        lda terminal_parse_v+1
+   jmp terminal_cursor_hor
    rts

; scroll up (SU)
terminal_csi_scroll_up:
    lda terminal_parse_v
    bne + ; default 1
        ina
+   eor #$ff    ; negate
    ina
    jmp terminal_scroll_down

; scroll down (SD)
terminal_csi_scroll_down:
    lda terminal_parse_v
    bne + ; default 1
        ina
+   jmp terminal_scroll_down

; vertical position absolute (VPA)
terminal_csi_cursor_vert_abs:
    lda terminal_parse_v
    jmp terminal_cursor_vert

; select graphics rendition (SGR)
terminal_csi_set_graphics:
    lda terminal_parse_v
    bne _not_0
        ; all attributes off
        stz terminal_parse_colour
        rts
_not_0:
    cmp #30
    bne _not_30
_fg_black:
        stz terminal_parse_colour
        rts
_not_30:
    cmp #31
    bne _not_31
        ; set foreground colour to red
        lda #64
        sta terminal_parse_colour
        rts
_not_31:
    ; remap all other foreground colours to black
    ; (this makes sure that if a program sets the colour to red, and then
    ; to another unsupported colour, the colour returns to black)
    cmp #32
    beq _fg_black
    cmp #33
    beq _fg_black
    cmp #34
    beq _fg_black
    cmp #35
    beq _fg_black
    cmp #36
    beq _fg_black
    cmp #37
    beq _fg_black
    cmp #38         ; set foreground 256 or true colour
    beq _fg_black
    cmp #39         ; default foreground colour
    beq _fg_black
    ; unsupported SGR parameter -- ignore it
    rts

; erase in display (ED)
; parameter specifies what part of the line is erased:
;   0: cursor - end of display
;   1: start of display - cursor
;   2: start of display - end of display
terminal_csi_erase_in_display:
    ; save initial cursor position
    lda vera_addr_mid
    d_push
    lda vera_addr_lo
    d_push                              ; ( Cursor )
    ; erase in line with the same parameter
    jsr terminal_csi_erase_in_line  
    ; then erase full lines
    ; if mode is not zero, start on first displayed line
    lda terminal_parse_v
    beq +
        lda terminal_top_line
        sta vera_addr_mid
+   ; if mode is one, end at cursor. otherwise end at last displayed line
    cmp #1
    bne +
        ldy 1,x                         ; Cursor hi (ie. row)
        bra ++
+   ; else
        lda terminal_top_line
        clc
        adc #30 ; top line + 30 = first off screen line
        and #$7f
        tay
+   ; .Y contains first row that is not cleared
    ; vera_addr_mid contains the first row that is cleared.
    ; .A will be used to loop over columns
_outer_loop:
    stz vera_addr_lo
    lda #20 ; loop twenty times over 8-bytes = 80*2 columns
_inner_loop:
    .rept 8
        stz vera_data0
    .next
    dea
    bne _inner_loop
    ; increment row
    lda vera_addr_mid
    ina
    and #$7f
    sta vera_addr_mid
    cpy vera_addr_mid
    bne _outer_loop
    ; end of loops
    ; if mode is zero then cursor returns to top left corner
    lda terminal_parse_v
    bne _restore_cursor
        stz vera_addr_mid
        stz vera_addr_lo
        bra _return
_restore_cursor:
    ; if mode is not zero, then return cursor to its previous position
        lda 0,x                             ; Cursor lo
        sta vera_addr_lo
        lda 1,x                             ; Cursor hi
        sta vera_addr_mid
_return:
    d_drop
    d_drop                                  ; ( )
    rts

; erase in line (EL)
; parameter specifies what part of the line is erased:
;   0: cursor - end of line
;   1: start of line - cursor
;   2: start of line - end of line
terminal_csi_erase_in_line:
    ; save hor position of cursor to stack
    lda vera_addr_lo
    d_push                                      ; ( cursor )
    ; if mode is not zero then start at beginning of line
    lda terminal_parse_v
    beq +
        stz vera_addr_lo
+
    ; if mode is 1 then end position is at cursor, otherwise 80*2
    cmp #1
    bne +
        lda 0,x
        bra ++
+   ; else
        lda #80*2
+   ; put data into vera until vera_addr_lo == .A
_loop:
    stz vera_data0
    stz vera_data0
    cmp vera_addr_lo
    bne _loop
    ; restore cursor position
    d_pop
    sta vera_addr_lo
    rts
