; push the accumulator onto the stack
; ( -- a )
d_push: .macro
    dex
    sta 0,x
.endm

; push 16-bit constant onto the stack
d_push16c: .macro
    lda #>\1
    d_push
    lda #<\1
    d_push
.endm

; push y onto the stack
; ( -- y )
d_push_y: .macro
    dex
    sty 0,x
.endm

; push zero onto the stack
; ( -- 0 )
d_push_zero: .macro
    dex
    stz 0,x
.endm

; pop the top value off the stack, into accumulator
; ( a -- )
d_pop: .macro
    lda 0,x
    inx
.endm

; pop the top value off the stack, into y
; ( y -- )
d_pop_y: .macro
    ldy 0,x
    inx
.endm

; pop the top value off the stack, but discard it
; ( a -- )
d_drop: .macro
    inx
.endm

; pop the top word off the stack and discard it
d_drop16: .macro
    inx
    inx
.endm

; load the top byte in the stack into the accumulator, but don't pop it
; ( a -- a )
d_peek: .macro
    lda 0,x
.endm

; duplicate the top byte on the stack
; ( a -- a a )
d_dup: .macro
    d_peek
    d_push
.endm

; duplicate the top word on the stack
; ( A -- A A )
d_dup16: .macro
    d_sdup
    d_sdup
.endm

; duplicate the second byte on the stack onto the top
; call it twice in a row to duplicate a 16-bit word
; ( b a -- b a b )
d_sdup: .macro
    lda 1,x
    d_push
.endm

; shift 16-bit number on the stack to the left
; ( hi lo -- hi' lo' )
d_asl16: .macro
    asl 0,x
    rol 1,x
.endm

; shift 16-bit number to the right with sign extend
; ( hi lo -- hi' lo' )
d_asr16: .macro
    .rept \1
        lda 1,x
        cmp #$80
        ror 1,x
        ror 0,x
    .next
.endm

; same 16-bit asr, but do it on second item in stack
; ( X Y -- X>> Y )
d_asr16_sub: .macro
    .rept \1
        lda 3,x
        cmp #$80
        ror 3,x
        ror 2,x
    .next
.endm

; shift 16-bit number on the stack to the right
; ( hi lo -- hi' lo' )
d_lsr16: .macro
    lsr 1,x
    ror 0,x
.endm

; add a 16-bit constant onto 16-bit value on top of stack
; ( hi lo -- hi' lo' )
d_adc16c: .macro
    lda 0,x ; lo
    adc #<\1
    sta 0,x
    lda 1,x ; hi
    adc #>\1
    sta 1,x
.endm

; add two 16 bit numbers, keep second operand on the stack
; ( A B -- A+B B )
d_adc16_accumulate: .macro
    lda 2,x
    adc 0,x
    sta 2,x

    lda 3,x
    adc 1,x
    sta 3,x
.endm

; add two 16 bit numbers from the stack
; ( A B -- A+B )
d_adc16: .macro
    d_adc16_accumulate
    d_drop
    d_drop
.endm

; logical not of 16 bit number on top of stack
; ( A -- ~A )
d_not16: .macro
    lda #$ff
    eor 0,x
    sta 0,x
    lda #$ff
    eor 1,x
    sta 1,x
.endm

; calculate 16 bit two's complement
; imma cheat though and not check the carry when +1
; ( A -- -A )
d_neg16: .macro
    d_not16
    inc 0,x
.endm


                                            ; routines for seccondary stacks
ds_push: .macro
    dec \1_index
    ldy \1_index
    sta \1,y
.endm

ds_pop: .macro
    ldy \1_index
    lda \1,y
    inc \1_index
.endm

ds_peek: .macro
    ldy \1_index
    lda \1,y
.endm

ds_drop: .macro
    inc \1_index
.endm
