.cpu "65c02"
.include "stack.s"
.include "defn.s"
*=$0801         ; Assembled code should start at $0801
            ; (where BASIC programs start)
            ; The real program starts at $0810 = 2064
.text $0C,$08       ; $080C - pointer to next line of BASIC code
.text $0A,$00       ; 2-byte line number ($000A = 10)
.text $9E       ; SYS BASIC token
.text $20       ; [space]
.text $32,$30,$36,$34   ; $32="2",$30="0",$36="6",$34="4"
            ; (ASCII encoded nums for dec starting addr)
.text $00       ; End of Line
.text $00,$00       ; This is address $080C containing
            ; 2-byte pointer to next line of BASIC code
            ; ($0000 = end of program)
*=$0810         ; Here starts the real program

reset:
    bra _code
_unicode_fname:
    .enc "none"
    .text "UNICODE.BIN"
_unifont_fname:
    .enc "none"
    .text "UNIFONT.BIN"
_pal_fname:
    .enc "none"
    .text "PAL.BIN"
_code:
    ; disable video so that junk isn't shown during loading
    stz vera_dc_video
    ; set iso mode for keyboard
    lda #$0f
    jsr chrout

    ; load unicode map
    lda #1 ; logical file no
    ldx #8 ; device number
    ldy #-1 ; secondary address
        jsr setlfs
    lda #11 ; length of filename
    ldx #<_unicode_fname
    ldy #>_unicode_fname
        jsr setnam
    lda #0
        jsr load

    ; load font glyphs
    lda #1
    ldx #8
    ldy #0
        jsr setlfs
    lda #11
    ldx #<_unifont_fname
    ldy #>_unifont_fname
        jsr setnam
    lda #2
    ldx #0
    ldy #0
        jsr load

    ; load palette
    lda #1
    ldx #8
    ldy #-1
        jsr setlfs
    lda #7
    ldx #<_pal_fname
    ldy #>_pal_fname
        jsr setnam
    lda #3
        jsr load

    ; fill up tilemap with zeroes
    lda #$11
    sta vera_addr_hi
    stz vera_addr_mid
    stz vera_addr_lo
    ; tilemap is 128 * 128 * 2 bytes = 32k
    ldy #16
_tilemap_outer_loop:
    ldx #$00
_tilemap_inner_loop:
    stz vera_data0
    stz vera_data0
    stz vera_data0
    stz vera_data0
    stz vera_data0
    stz vera_data0
    stz vera_data0
    stz vera_data0
    dex
    bne _tilemap_inner_loop
    dey
    bne _tilemap_outer_loop

    ; set up vera registers
    lda #$a2
    sta vera_l0_config
    lda #$80            ; map base is $10000
    sta vera_l0_mapbase
    lda #$02
    sta vera_l0_tilebase
    lda #$53
    sta vera_dc_video   ; vga video; layer 0, sprites visible

    ; set up vera address for cursor
    lda #$11
    sta vera_addr_hi
    stz vera_addr_mid
    stz vera_addr_lo
    
    ; set up interrupt handler
    lda $0314
    sta kernal_interrupt_handler
    lda $0315
    sta kernal_interrupt_handler+1
    ; replace with custom handler
    lda #<interrupt_handler
    sta $0314
    lda #>interrupt_handler
    sta $0315

    ; data stack pointer
    ldx #$80
    cli

    ; do some stuff
    ; initialize terminal
    stz terminal_parse_state 
    stz terminal_parse_colour
    stz terminal_top_line
_loop:
    ; handle input
    lda keyboard_buffer
    beq _no_keyboard_buffer
        stz keyboard_buffer
        cmp #$14
        bne +           ; backspace
            lda #8  ; ascii backspace
            jsr serial_chrout
            bra _no_keyboard_buffer
+       cmp #$11
        bne +           ; cursor down
            lda #$1b ; escape
            jsr serial_chrout
            lda #'['
            jsr serial_chrout
            lda #'B'
            jsr serial_chrout
            bra _no_keyboard_buffer
+       cmp #$91        ; cursor up
        bne +
            lda #$1b
            jsr serial_chrout
            lda #'['
            jsr serial_chrout
            lda #'A'
            jsr serial_chrout
            bra _no_keyboard_buffer
+       cmp #$1d        ; cursor right
        bne +
            lda #$1b
            jsr serial_chrout
            lda #'['
            jsr serial_chrout
            lda #'C'
            jsr serial_chrout
            bra _no_keyboard_buffer
+       cmp #$9d        ; cursor left
        bne +
            lda #$1b
            jsr serial_chrout
            lda #'['
            jsr serial_chrout
            lda #'D'
            jsr serial_chrout
            bra _no_keyboard_buffer
+       cmp #$83        ; escape
        bne +
            lda #$1b
            jsr serial_chrout
+
        jsr serial_chrout
_no_keyboard_buffer:
    ; handle output
    jsr serial_chrin
    ora #0
    beq _loop
    jsr terminal_insertch
    bra _loop

interrupt_handler:
    ; check for keyboard input
    lda keyboard_buffer
    bne +
        ; handle keyboard input
        jsr clrchn
        jsr getin
        sta keyboard_buffer
    +
    ; set vera_l0_vscroll
    lda terminal_top_line
    sta vera_l0_vscroll_lo
    stz vera_l0_vscroll_hi
    ; 16-bit shift left 4 times
    .rept 4
        asl vera_l0_vscroll_lo
        rol vera_l0_vscroll_hi
    .next
    ; pass off control to kernel interrupt handler
    jsr terminal_vera_show_cursor
    jmp (kernal_interrupt_handler)

kernal_interrupt_handler:
    .word 0
keyboard_buffer:
    .text 0

.include "unicode.s"
.include "utf8.s"
.include "emu_serial.s"
.include "terminal.s"
.include "csi_dispatch.s"
